"""Remote control Samsung televisions via TCP/IP connection"""

from .remote import Remote

__title__ = "samsungctl"
__version__ = "0.7.1"
__url__ = "https://github.com/Ape/samsungctl"
__author__ = "Lauri Niskanen"
__author_email__ = "ape@ape3000.com"
__license__ = "MIT"
